﻿using FontAwesome.WPF;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;

/*
 *         Progetto: Progetto "Mappa Meteo"
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 16/12/2016
 *  
 *      Descrizione: Definizione della classe e dei metodi di generazione della struttura
 *                   dati contenente le temperature da visualizzare.
 *                   
 */

namespace MappaMeteo
{
    [Serializable]
    public class StrutturaDati
    {
        private DateTime data;
        public DateTime Data { get { return data; } }
        public int TemperaturaMax { get; set; }
        public int TemperaturaMin { get; set; }

        private List<InfoCitta> citta;
        public List<InfoCitta> Citta { get { return citta; } }

        public double LatCentrale
        {
            // Latitudine del centro calcolata come media dei soli valori estremi (massimo e minimo)
            get
            {
                double max = Double.MinValue;
                double min = Double.MaxValue;
                foreach (var c in citta)
                {
                    if (c.Lat > max)
                        max = c.Lat;
                    if (c.Lat < min)
                        min = c.Lat;
                }

                return (max + min) / 2.0;
            }
        }

        public double LatMedia
        {
            // Latitudine del centro calcolata come media delle latitudini delle singole citta'
            get
            {
                double ris = 0;
                foreach (var c in citta)
                    ris += c.Lat;

                return ris / citta.Count();
            }
        }

        public double LongCentrale
        {
            // Longitudine del centro calcolata come media dei soli valori estremi (massimo e minimo)
            get
            {
                double max = Double.MinValue;
                double min = Double.MaxValue;
                foreach (var c in citta)
                {
                    if (c.Long > max)
                        max = c.Long;
                    if (c.Long < min)
                        min = c.Long;
                }

                return (max + min) / 2.0;
            }
        }

        public double LongMedia
        {
            // Longitudine del centro calcolata come media delle longitudini delle singole citta'
            get
            {
                double ris = 0;
                foreach (var c in citta)
                    ris += c.Long;

                return ris / citta.Count();
            }
        }

        public StrutturaDati(List<Point> clip, TextBlock tb = null, ImageAwesome pb = null)
        {
            this.data = DateTime.Now;
            this.TemperaturaMax = Int32.MinValue;
            this.TemperaturaMin = Int32.MaxValue;

            if (tb != null) tb.Dispatcher.Invoke(() => { tb.Text = "Lettura delle temperature in corso..."; });
            if (pb != null) pb.Dispatcher.Invoke(() =>
            {
                pb.Visibility = System.Windows.Visibility.Visible;
            });


            this.citta = StrutturaDati.LeggiDatiCitta(pb, clip).Result;
            if (pb != null) pb.Dispatcher.Invoke(() => { pb.Visibility = System.Windows.Visibility.Collapsed; });
            if (tb != null) tb.Dispatcher.Invoke(() => { tb.Text = String.Format("Acquisite temperature di {0} città", this.Citta.Count); });

            foreach (var c in this.citta)
            {
                if (c.TemperaturaCelsius > this.TemperaturaMax)
                    this.TemperaturaMax = c.TemperaturaCelsius;
                if (c.TemperaturaCelsius < this.TemperaturaMin)
                    this.TemperaturaMin = c.TemperaturaCelsius;

            }
        }

        public static async Task<StrutturaDati> GeneraAsync(List<Point> clip, TextBlock tb = null, ImageAwesome pb = null)
        {

            if (Cache.OggettoInCache())
            {
                var sd = (StrutturaDati)Cache.Carica();
                if (tb != null)
                    tb.Dispatcher.Invoke(() => { tb.Text = "Dati caricati dalla cache locale"; });
                return sd;

            }

            StrutturaDati ris = null;
            await Task.Run(() =>
            {
                ris = new StrutturaDati(clip, tb, pb);
            }
            );

            return ris;
        }

        private static bool EsisteCittaVicina(List<InfoCitta> lista, InfoCitta c)
        {
            const double distMin = 0.55;

            foreach (var e in lista)
            {
                double dist = Math.Sqrt(Math.Pow(c.Long - e.Long, 2) + Math.Pow(c.Lat - e.Lat, 2));

                if (dist < distMin)
                    return true;
            }

            return false;

        }

        private static bool CittaInPoligono(List<Point> poligono, InfoCitta citta)
        {
            int i, j;
            bool c = false;


            for (i = 0, j = poligono.Count - 1; i < poligono.Count; j = i++)
            {
                if (((poligono[i].Y > citta.Lat) != (poligono[j].Y > citta.Lat)) &&
                 (citta.Long < (poligono[j].X - poligono[i].X) * (citta.Lat - poligono[i].Y) / (poligono[j].Y - poligono[i].Y) + poligono[i].X))
                    c = !c;
            }
            return c;
        }

        static void Clip2BBox(List<Point> clip, out Double LonLeft, out Double LatBottom, out Double LonRight, out Double LatTop)
        {
            LonLeft = Double.PositiveInfinity;
            LatBottom = Double.PositiveInfinity;
            LonRight = Double.NegativeInfinity;
            LatTop = Double.NegativeInfinity;

            foreach (var e in clip)
            {
                if (e.X < LonLeft) LonLeft = e.X;
                if (e.X > LonRight) LonRight = e.X;
                if (e.Y < LatBottom) LatBottom = e.Y;
                if (e.Y > LatTop) LatTop = e.Y;
            }
        }

        private static async Task<List<InfoCitta>> LeggiDatiCitta(ImageAwesome pb, List<Point> poligono)
        {
            Double LonLeft;
            Double LatBottom;
            Double LonRight;
            Double LatTop;
            int Zoom = 8;

            Clip2BBox(poligono, out LonLeft, out LatBottom, out LonRight, out LatTop);

            var lista = new List<InfoCitta>();
            int limite = -1;


            OpenWeatherMapRequest.ApiKey = Properties.Settings.Default.ApiKey;
            var result = await OpenWeatherMapRequest.GetCitiesByZone(LonLeft, LatBottom, LonRight, LatTop, Zoom);
            if (!result.Success)
                return lista;

            foreach (var citta in result.Items)
            {
                if (limite == 0)
                    break;
                try
                {
                    InfoCitta c = new InfoCitta();
                    c.Successo = true;
                    c.Localita = citta.City;
                    c.Data = citta.Date;
                    c.TemperaturaCelsius = Convert.ToInt32(citta.Temp);
                    c.Lat = citta.Lat;
                    c.Long = citta.Lon;
                    c.MeteoTitolo = citta.Title;
                    c.MeteoDescrizione = citta.Description;
                    c.MeteoIcona = citta.Icon;

                    if ((EsisteCittaVicina(lista, c) == false) && (CittaInPoligono(poligono, c) == true))
                    {
                        lista.Add(c);
                        limite--;
                    }
                }
                catch (Exception) { }
            }

            return lista;
        }
    }
}
